/** @file
===========================================================================
MODULE          : directshowsamplegrabber

DESCRIPTIONS    : 

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#pragma once

// Needed for sample grabber
#include <qedit.h>
//
// Implementation of CSampleGrabberCB object
//
// Note: this object is a SEMI-COM object, and can only be created statically.

// Live media includes
#include "liveMedia.hh"
#include "GroupsockHelper.hh"
#include "BasicUsageEnvironment.hh"

// DirectShow LiveMedia Integration
#include <DirectShow/LiveIntegration/PollingTaskScheduler.h>
#include <DirectShow/LiveIntegration/DirectShowDeviceSource.h>
#include <DirectShow/LiveIntegration/audiodirectshowdevicesource.h>

class CDirectShowSampleGrabberCB : public ISampleGrabberCB 
{

public:

	CDirectShowSampleGrabberCB(AM_MEDIA_TYPE* pMediaType)
		: m_pDirectShowDeviceSource(NULL),
		m_pMediaType(pMediaType)
	{}

	~CDirectShowSampleGrabberCB()
	{
		//TOREVISE: Do we need to free this type???
		if (m_pMediaType)
		{
			FreeMediaType(*m_pMediaType);
		}
	}

	void InitialiseLive555Source(BasicUsageEnvironment* pEnv, PollingTaskScheduler* pScheduler)
	{
		if (m_pMediaType->majortype == MEDIATYPE_Audio)
		{
			if (m_pMediaType->subtype == MEDIASUBTYPE_PCM)
			{
				if (m_pMediaType)
				{
					WAVEFORMATEX* pWavHeader = (WAVEFORMATEX*)m_pMediaType->pbFormat;
					WavHeaderData* pHeaderInfo = new WavHeaderData(pWavHeader->wFormatTag, pWavHeader->wBitsPerSample, pWavHeader->nSamplesPerSec, pWavHeader->nChannels);
					m_pDirectShowDeviceSource = CAudioDirectShowDeviceSource::createNew(*pEnv, pHeaderInfo);
					pScheduler->registerObservableSource(m_pDirectShowDeviceSource);
				}
			}
		}
		else if (m_pMediaType->majortype == MEDIATYPE_Video)
		{
			m_pDirectShowDeviceSource = CDirectShowDeviceSource::createNew(*pEnv);
			pScheduler->registerObservableSource(m_pDirectShowDeviceSource);
		}
	}

	// Fake out any COM ref counting
	//
	STDMETHODIMP_(ULONG) AddRef() { return 2; }
	STDMETHODIMP_(ULONG) Release() { return 1; }

	// Fake out any COM QI'ing
	//
	STDMETHODIMP QueryInterface(REFIID riid, void ** ppv)
	{
		CheckPointer(ppv,E_POINTER);

		if( riid == IID_ISampleGrabberCB || riid == IID_IUnknown ) 
		{
			*ppv = (void *) static_cast<ISampleGrabberCB*> ( this );
			return NOERROR;
		}    

		return E_NOINTERFACE;
	}


	// We implement this one
	// Note this all happens in the DirectShow streaming thread!!!!!!!!!!!!
	STDMETHODIMP SampleCB( double SampleTime, IMediaSample * pSample )
	{
		if (m_pDirectShowDeviceSource)
		{
			CMediaFrame* pMediaFrame = CMediaFrame::CreateNew(pSample);
			m_pDirectShowDeviceSource->AddFrame(pMediaFrame);
		}
		return 0;
	}

	// The sample grabber is calling us back on its deliver thread.
	// This is NOT the main app thread!
	//
	STDMETHODIMP BufferCB( double SampleTime, BYTE * pBuffer, long BufferSize )
	{
		return 0;
	}

	void DirectShowDeviceSource(CDirectShowDeviceSource* val) { m_pDirectShowDeviceSource = val; }
	CDirectShowDeviceSource* DirectShowDeviceSource() { return m_pDirectShowDeviceSource; }

private:
	CDirectShowDeviceSource* m_pDirectShowDeviceSource;
	AM_MEDIA_TYPE* m_pMediaType;
	BasicUsageEnvironment* m_pEnvironment;
	PollingTaskScheduler* m_pScheduler;
};
