/** @file
===========================================================================
MODULE          : directshowdevicesource

DESCRIPTIONS    : 

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#ifndef __DIRECT_SHOW_DEVICE_SOURCE__H
#define __DIRECT_SHOW_DEVICE_SOURCE__H

// STL
#include <Queue> // Queue for frames

// DirectShow
#include <wxutil.h> // LOCKING

// LiveMedia
#include <FramedSource.hh>

// DirectShow LiveMedia Integration
#include "MediaFrame.h"
#include "IScheduledTask.h"

// Const time factor between LiveMedia and DirectShow
const int DIRECT_SHOW_TOLIVE_TIME_FACTOR = 10;


class CDirectShowDeviceSource : public IScheduledTask, public FramedSource
{
public:
	static CDirectShowDeviceSource* createNew(UsageEnvironment& env);

	// Static method needed to process queued events in the event scheduler
	static void doDeliverFrame(void* pInstance);
	
	// method to add data to the device
	void AddFrame(CMediaFrame* pMediaFrame);

	virtual void setScheduledTask(){ m_fnScheduledTask = &(CDirectShowDeviceSource::doDeliverFrame);}

	bool hasNewData()
	{
		CAutoLock lock(&m_Lock);
		return (m_qFrameQueue.size()>0);
	}
protected:

	CDirectShowDeviceSource(UsageEnvironment& env);
	virtual ~CDirectShowDeviceSource();


private:
	// redefined virtual functions:
	virtual void doGetNextFrame();

private:
	//TODO Should this be virtual?
	void deliverFrame();

private:
	// Queue and lock
	std::queue<CMediaFrame*> m_qFrameQueue;
	CCritSec m_Lock;
};
#endif
