/** @file
===========================================================================
MODULE          : DirectShowHelper

DESCRIPTIONS    : Static DirectShow helper methods ripped from the examples

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#pragma once

#include <streams.h>
#include <vector>
#include <string>

typedef std::vector<std::string> FILTER_NAME_LIST ;

class CDirectShowHelper
{
public:
	CDirectShowHelper();

	// Find an unconnected pin on a filter
	// This too is stolen from the DX9 SDK
	static HRESULT GetUnconnectedPin(IBaseFilter *pFilter, PIN_DIRECTION PinDir, IPin **ppPin);


	// Connect two filters together with the filter graph manager
	// Stolen from the DX9 SDK
	// This is the base version
	static HRESULT ConnectFilters(IGraphBuilder *pGraph, IPin *pOut, IBaseFilter *pDest);

	// Connect two filters together with the filter graph manager
	// Again, stolen from the DX9 SDK
	// This is an overloaded version
	static HRESULT ConnectFilters(IGraphBuilder *pGraph, IBaseFilter *pSrc, IBaseFilter *pDest);

	//Helper function to retrieve filters of a certain type
	static HRESULT GetFilterByCategory(IBaseFilter** gottaFilter, LPCWSTR matchName, GUID Category);
	
	static HRESULT GetFilterByCategory(IBaseFilter** gottaFilter, std::string sMatchName, GUID Category);

	// This code was also brazenly stolen from the DX9 SDK
	// Pass it a file name in wszPath, and it will save the filter graph to that file.
	static HRESULT SaveGraphFile(IGraphBuilder *pGraph, WCHAR *wszPath);

	// A very useful bit of code 
	// Stolen from the DX9 SDK
	static HRESULT AddFilterByCLSID(IGraphBuilder *pGraph, const GUID& clsid, LPCWSTR wszName, IBaseFilter **ppF);
	//static HRESULT AddFilterByCLSID(IGraphBuilder *pGraph, const GUID& clsid, std::string sName, IBaseFilter **ppF);

	static HRESULT AddFilterByCategoryAndName(IGraphBuilder *pGraph, const GUID& category, LPCWSTR wszName, IBaseFilter **ppF);
	static HRESULT AddFilterByCategoryAndName(IGraphBuilder *pGraph, const GUID& category, std::string sName, IBaseFilter **ppF);

	// Show the property pages for a filter
	// This is stolen from the DX9 SDK
	HRESULT ShowFilterPropertyPages(IBaseFilter *pFilter);

	static HRESULT GetFilterNamesByCategory(GUID Category, FILTER_NAME_LIST& vList);
	static HRESULT GetVideoInputs(FILTER_NAME_LIST& vList);
	static HRESULT GetAudioInputs(FILTER_NAME_LIST& vList);

	static HRESULT GetPin(IBaseFilter *pFilter, PIN_DIRECTION PinDir, IPin **ppPin);
};
