/** @file

MODULE				: RealYUV420toRGB24Converter

TAG					: RYUVRGB24C

FILE NAME			: RealYUV420toRGB24Converter.h

DESCRIPTION			: Floating point implementation of YUV420 (8 bpp) to RGB 24 bit 
					  colour convertion derived from the YUV420toRGBConverter base 
					  class.

REVISION HISTORY	:

COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
					  the property of CSIR, Meraka Institute and has been 
					  classified as PUBLIC.
===========================================================================
*/
#ifndef _REALYUV420TORGB24CONVERTER_H
#define _REALYUV420TORGB24CONVERTER_H

#include "YUV420toRGBConverter.h"

/*
===========================================================================
  Class definition.
===========================================================================
*/
class RealYUV420toRGB24Converter: public YUV420toRGBConverter
{
	public:
		// Construction and destruction.
		RealYUV420toRGB24Converter(void) { }
		RealYUV420toRGB24Converter(int width, int height): YUV420toRGBConverter(width,height) { }
		virtual ~RealYUV420toRGB24Converter(void) {}

		// Interface.
		virtual void Convert(void* pY, void* pU, void* pV, void* pRgb) 
		{
			if(_rotate) RotateConvert(pY, pU, pV, pRgb);
			else				NonRotateConvert(pY, pU, pV, pRgb);
		};

	protected:
		virtual void NonRotateConvert(void* pY, void* pU, void* pV, void* pRgb);
		virtual void RotateConvert(void* pY, void* pU, void* pV, void* pRgb);

};//end RealYUV420toRGB24Converter.


#endif	// _REALYUV420TORGB24CONVERTER_H
